using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using ParallelCPP;

namespace TestFor
{
  class Program
  {
    static void Main(string[] args)
    {
      const int iAnz = 10000000;
      double[] arr = new double[iAnz];

      Stopwatch sw = new Stopwatch();

      // Parallel

      sw.Start();

      ParExec.For(0, iAnz, delegate(int i)
      {
        arr[i] = Math.Sqrt(i) + Math.Pow(i, 0.3);
      });
      
      sw.Stop();
      long timePar = sw.ElapsedTicks;
      sw.Reset();

      // Serial

      sw.Start();

      for (int i = 0; i < iAnz; i++)
      {
        arr[i] = Math.Sqrt(i) + Math.Pow(i, 0.3);
      }

      sw.Stop();
      long timeSer = sw.ElapsedTicks;
      sw.Reset();

      // Test

      double d = 0.0;
      double dTest = 0;

      for (int i = 0; i < iAnz; i++)
      {
        d += arr[i];
        dTest += Math.Sqrt(i) + Math.Pow(i, 0.3); ;
      }

      Console.WriteLine("Summe: {0}   Test: {1}", d, dTest);
      Console.WriteLine("Time parallel: {0}   Time serial: {1}", timePar, timeSer);

      // Reduction parallel

      sw.Start();

      double dSum = ParExec.ForReduceDouble(0, iAnz, 0.0, Reduction.Add, delegate(int i)
      {
        return Math.Sqrt(i) + Math.Pow(i, 0.3);
      });

      sw.Stop();
      timePar = sw.ElapsedTicks;
      sw.Reset();

      // Reduction serial

      sw.Start();

      double dSum2 = 0.0;

      for (int i = 0; i < iAnz; i++)
      {
        dSum2 += Math.Sqrt(i) + Math.Pow(i, 0.3);
      }

      sw.Stop();
      timeSer = sw.ElapsedTicks;
      sw.Reset();

      Console.WriteLine("Summe: {0}     Test: {1}", dSum, dSum2);
      Console.WriteLine("Time parallel: {0}   Time serial: {1}", timePar, timeSer);
    }
  }
}
